// Initialize the origins log array
let originsLog = [];

// Ensure global.healthStore is properly initialized
if (!global.healthStore) {
    global.healthStore = {};
}

// Function to add an event to the log
function logOriginEvent(player, eventType, message) {
    const event = {
        date: new Date().toISOString(),
        eventType: eventType,
        message: message,
        player: player.getName().getString()
    };
    
    originsLog.push(event);
    
    // Write the updated log to the file
    JsonIO.write("kubejs/data/origins.log", originsLog);
    
    // Log the event to the console
    console.log(`Origin Event: ${eventType} - ${message} - ${player.getName().getString()}`);
}

// Function to get the origin of a player
const getOrigin = (player) => {
    const nbt = player.getNbt();
    if (nbt.ForgeCaps && nbt.ForgeCaps['origins:origins']) {
        const originsData = nbt.ForgeCaps['origins:origins'];
        const origin = originsData.Origins['origins:origin'];
        return origin;
    }
    return null;
};

// Function to handle origin token usage
function handleOriginToken(player, item) {
    const nbt = item.nbt;
    const server = player.getServer();
    if (nbt && nbt.Origin) {
        // Set the origin to the one specified in the NBT
        server.runCommandSilent(`origin set ${player.getName().getString()} origins:origin ${nbt.Origin}`);
    } else {
        // Randomly set the origin if no NBT is present
        server.runCommandSilent(`origin random ${player.getName().getString()} origins:origin`);
        // Schedule the announcement
        server.scheduleInTicks(20, () => {
            const newOrigin = getOrigin(player);
            if (newOrigin) {
                server.runCommandSilent(`tellraw @a {"text":"§f${player.getName().getString()}§7 has rolled the origin: §l§d${prettifyOriginName(newOrigin)}§r"}`);
            }
        });
    }
}

ServerEvents.commandRegistry(event => {
    const { commands: Commands, arguments: Arguments } = event;
    
    event.register(
        Commands.literal('check-origin')
            .executes(ctx => {
                const player = ctx.source.player;
                if (!player) {
                    return 'This command must be run by a player!';
                }
        
                const nbt = player.getNbt();
                
                if (nbt.ForgeCaps && nbt.ForgeCaps['origins:origins']) {
                    const originsData = nbt.ForgeCaps['origins:origins'];
                    
                    const origin = originsData.Origins['origins:origin'];
                    
                    if (origin) {
                        ctx.source.sendSuccess(Text.of(`Your origin is: ${prettifyOriginName(origin)}`), false);
                        logOriginEvent(player, 'CHECK_ORIGIN', `Player checked their origin: ${prettifyOriginName(origin)}`);
                        return 1; 
                    } else {
                        ctx.source.sendSuccess(Text.of('You don\'t have an origin set.'), false);
                        logOriginEvent(player, 'CHECK_ORIGIN', 'Player checked their origin, but no origin was set');
                        return 0; 
                    }
                } else {
                    ctx.source.sendSuccess(Text.of('Origins data not found.'), false);
                    logOriginEvent(player, 'CHECK_ORIGIN', 'Player checked their origin, but origins data was not found');
                    return 0; 
                }
            })
    );

    event.register(
        Commands.literal('withdraw-origin')
            .executes(ctx => {
                const player = ctx.source.player;
                if (!player) {
                    return 'This command must be run by a player!';
                }
                
                const uuid = player.uuid.toString();

                // Ensure the player's health data is initialized in healthStore
                if (!global.healthStore[uuid]) {
                    const persistentHealth = player.getPersistentData().getInt('lifestealHealth') || inverseNonLinearMapping(player.getMaxHealth());
                    global.healthStore[uuid] = persistentHealth;
                }

                const nbt = player.getNbt();
                if (nbt.ForgeCaps && nbt.ForgeCaps['origins:origins']) {
                    const originsData = nbt.ForgeCaps['origins:origins'];
                    const origin = originsData.Origins['origins:origin'];
                    
                    if (origin && origin !== 'origins:human') {
                        if((global.healthStore[uuid] - 1) < global.MIN_HEALTH) {
                            ctx.source.sendSuccess(Text.of(`You don't have enough health to withdraw your origin.`), false);
                            logOriginEvent(player, 'WITHDRAW_ORIGIN_FAILED', `Player attempted to withdraw origin but didn't have enough health`);
                            return 0;
                        }

                        global.UpdateHealth(player, -1);
                        // Create the origin token with the current origin
                        const originToken = Item.of('kubejs:origin_token');
                        originToken.nbt = { Origin: origin };
        
                        originToken.setHoverName(Text.of(`Origin Token`).bold());
                        Utils.server.runCommandSilent(`origin set ${player.getName().getString()} origins:origin origins:human`);

                        // Give the token to the player
                        player.give(originToken);
                        ctx.source.sendSuccess(Text.of(`You have turned back human. Given Origin Token for ${prettifyOriginName(origin)}`), false);
                        logOriginEvent(player, 'WITHDRAW_ORIGIN', `Player withdrew their origin: ${prettifyOriginName(origin)}`);
                        return 1;
                    } else {
                        ctx.source.sendSuccess(Text.of('You don\'t have an origin set.'), false);
                        logOriginEvent(player, 'WITHDRAW_ORIGIN_FAILED', 'Player attempted to withdraw origin, but no origin was set');
                        return 0;
                    }
                } else {
                    ctx.source.sendSuccess(Text.of('Origins data not found.'), false);
                    logOriginEvent(player, 'WITHDRAW_ORIGIN_FAILED', 'Player attempted to withdraw origin, but origins data was not found');
                    return 0;
                }
            })
    );
});

// Helper function to prettify the origin name
function prettifyOriginName(origin) {
    return origin.split(':')[1].replace('_', ' ').toUpperCase();
}

// Load existing log if it exists
const existingLog = JsonIO.read("kubejs/config/origins-log.json");
if (existingLog) {
    originsLog = existingLog;
}

// Handle shift-clicking the origin token
ItemEvents.rightClicked('kubejs:origin_token', event => {
    if (event.player.isCrouching()) {
        handleOriginToken(event.player, event.item);
        event.item.count--;
    }
});
