// Global tracking data store
global.trackingData = new Map();

function generateUUID() {
    let d = new Date().getTime();
    let d2 = ((typeof performance !== 'undefined') && performance.now && (performance.now() * 1000)) || 0;
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
        let r = Math.random() * 16;
        if (d > 0) {
            r = (d + r) % 16 | 0;
            d = Math.floor(d / 16);
        } else {
            r = (d2 + r) % 16 | 0;
            d2 = Math.floor(d2 / 16);
        }
        return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
}


// Event for right-clicking an entity with the player tracker
ItemEvents.entityInteracted('kubejs:tracker', event => {
    const { item, player, target, level } = event;

    const trackerUUID = item.nbt?.trackerUUID || generateUUID();
    console.log(trackerUUID);
    const targetUUID = target.uuid;

    player.tell([Text.of(`Started tracking entity: `), Text.lightPurple(`${target.getName().getString()}`).bold()]);

 
    // Store tracking data server-side
    global.trackingData.set(trackerUUID, {
        targetUUID: targetUUID,
        targetName: target.getName().getString(),
        dimension: level.dimension.toString(),
        x: target.x,
        y: target.y,
        z: target.z
    });

    item.setNbt({ trackerUUID: trackerUUID});
    item.setHoverName(Text.of(`§rTracking: §r§l${target.getName().getString()}`));

    // Play a sound to indicate successful tracking (optional)
    target.playSound('minecraft:entity.experience_orb.pickup');
})

// Event for handling shift-click on the player tracker
ItemEvents.rightClicked('kubejs:tracker', event => {
    const { item, player } = event;
    
    // Check if the player is sneaking (shift-clicking)
    if (player.isCrouching()) {
        const trackerUUID = item.nbt?.trackerUUID;
        if (trackerUUID) {
            const data = global.trackingData.get(trackerUUID);
            if (data) {
                // Send a message to the player with the tracking details
                player.tell(Text.of([
                    `§6Tracking: §r${data.targetName}\n`,
                    `§6Location: §r${data.dimension}\n`,
                    `§6Coordinates: §r(${data.x.toFixed(2)}, ${data.y.toFixed(2)}, ${data.z.toFixed(2)})`
                ].join('')));
                
                // Cancel the event to prevent normal right-click behavior
                event.cancel();
            }
        }
    }
})

// Events for player login and logout
PlayerEvents.loggedIn(event => handlePlayerLoginLogout(event.player, 'logged in'));
PlayerEvents.loggedOut(event => handlePlayerLoginLogout(event.player, 'logged out'));

function handlePlayerLoginLogout(player, action) {
    const playerUUID = player.uuid;

    // Remove tracking data if this player was being tracked
    for (const [trackerUUID, data] of global.trackingData.entries()) {
        if (data.targetUUID === playerUUID) {
            global.trackingData.delete(trackerUUID);
            notifyTrackerHolders(trackerUUID, `${data.targetName} has ${action}. Tracking stopped.`);
        }
    }

    // Clear tracking for items in the player's inventory
    player.inventory.allItems.forEach(item => {
        if (item.id === 'kubejs:tracker' && item.nbt) {
            const trackerUUID = item.nbt.getString('trackerUUID');
            if (global.trackingData.has(trackerUUID)) {
                global.trackingData.delete(trackerUUID);
                item.setNbt({});
                item.setHoverName(Text.lightPurple('Tracker'));
            }
        }
    });
}

function notifyTrackerHolders(trackerUUID, message) {
    // Notify all online players who might be holding the tracker
    Utils.server.getPlayers().forEach(player => {
        player.inventory.allItems.forEach(item => {
            if (item.id === 'kubejs:tracker' && item.nbt && item.nbt.trackerUUID === trackerUUID) {
                item.setNbt({});
                item.setCustomName(Text.of('Player Tracker'));
                player.tell(message);
            }
        });
    });
}

ServerEvents.tick(event => {
    // Only update every 20 ticks (1 second) to reduce performance impact
    if (event.server.getLevel('overworld').getTime() % 20 !== 0) return;

    for (const [trackerUUID, data] of global.trackingData.entries()) {
        // Try to get the entity from the server
        let discoveredEntity = null;
        event.server.getEntities().forEach(e => {
            if (e.uuid === data.targetUUID) {
                discoveredEntity = e;
            }
        });

        if (!discoveredEntity) {
            discoveredEntity = event.server.getPlayerList().getPlayer(data.targetUUID);
        }

        if (discoveredEntity) {
            // Entity is loaded, update its position
            data.dimension = discoveredEntity.level.dimension.toString();
            data.x = discoveredEntity.x;
            data.y = discoveredEntity.y;
            data.z = discoveredEntity.z;
            data.lastUpdated = Date.now();
        } else {
            console.log('Entity not loaded');
        }
    }

    // Update tracker items in player inventories
    event.server.getPlayerList().getPlayers().forEach(player => {
        player.getInventory().getAllItems().forEach(item => {
            if (item.getId() === 'kubejs:tracker') {
                const trackerUUID = item.getNbt()?.trackerUUID;
                if (trackerUUID && global.trackingData.has(trackerUUID)) {
                    updateTrackerItemNBT(item, global.trackingData.get(trackerUUID));
                }
                else if(trackerUUID) {
                    item.setNbt({});
                }
            }
        });
    });
});

// Function to update the tooltip of a tracker item
function updateTrackerItemNBT(item, data) {
    const trackingNbt = item.getNbt().getCompound('tracking_data');
    trackingNbt.putString('targetName', data.targetName);
    trackingNbt.putString('dimension', data.dimension);
    trackingNbt.putDouble('x', data.x);
    trackingNbt.putDouble('y', data.y);
    trackingNbt.putDouble('z', data.z);
    trackingNbt.putLong('lastUpdated', data.lastUpdated);
    item.getNbt().put('tracking_data', trackingNbt);
}
